// LCD.c
#include "GameGirl.h"		// inclure la librairie GameGirl.h
#include <p18f452.h>		// librairie du PIC
#include <delays.h>			// librairie de temporisation
#include "font_5x7.h"			// librairie des caractres

void LCD_Write_Command(unsigned char Data)
{	LCD_RS=0;				// dsactiver RS
	LCD_RW=0;				// dsactiver RW
	LCD_DATA=Data;			// placement de la donne sur le PORT
	Delay100TCYx(2);		// pendant 200 cyles
	LCD_Enable=1;			// activer Enable
	Delay100TCYx(2);		// pendant 200 cycles
	LCD_Enable=0;			// dsactiver Enable
}

void LCD_Init(void)
{	LCD_CSA=1;LCD_CSB=1;	// slection des deux contrleurs
	LCD_Enable=0;			// dsactiver Enable
	LCD_Reset=0;			// mettre Reset  0
	Delay100TCYx(1);		// pendant 100 cycles
	LCD_Reset=1;			// mettre Reset  1

	LCD_Write_Command(0x3F);	// instruction pour allumer l'afficheur
}

void LCD_Send_Data(unsigned char Data)
{	LCD_RS=1;				// activer RS
	LCD_RW=0;				// dsactiver RW
	LCD_DATA=Data;			// placement de la donne sur le PORT
	Delay100TCYx(2);		// pendant 200 cyles
	LCD_Enable=1;			// activer Enable
	Delay100TCYx(2);		// pendant 200 cycles
	LCD_Enable=0;			// dsactiver Enable
}

void LCD_Clear(void)
{	unsigned char i,x;
	LCD_CSA=1;						// slection du contrleur A
	LCD_CSB=1;						// slection du contrleur B
	
	for (x=0;x!=8;x++)			// rpter pour les 7 pages
	{
	LCD_Write_Command(0xb8 + x);	// slection de la page
	
		for (i=0;i!=64;i++)		// rpter pour les 64 octets
		LCD_Send_Data(0x00);	// envoyer un octet nul
	}
}

void LCD_GotoXY(unsigned char X,unsigned char Y)
{	if (X<64)
	{	
		LCD_CSA=0;
		LCD_CSB=1;			// slection du controleur B
		LCD_Write_Command(0xB8 + Y);	// slection de la page
		LCD_Write_Command(0x40 + X);	// slection de l'octet
	}
	else
	{	
		LCD_CSA=1;
		LCD_CSB=0;
		LCD_Write_Command(0xB8 + Y);
		LCD_Write_Command((0x40 + X)-64);
	}
}

void LCD_Putchar(char ch)
{ char i;
	for(i=0;i!=5;i++)
		LCD_Send_Data(Font[((int)(ch-32)*5)+i]);	// Envoyer le caratre ASCII
		LCD_Send_Data(0x00);				// Envoyer un octet vide pour l'espace
	Delay10KTCYx(10);		// tempo
}

char LCD_ReadData(void)
{ unsigned char data;
	TRISD = 0b11111111;		// le PortD est mis en entre
	LCD_RS=1;
	LCD_RW=1;
	LCD_Enable=1;
	Delay10TCYx(5);
	LCD_Enable=0;
	data=LCD_DATA;			// Lire la donne sur le port
	TRISD = 0b00000000;		// le PortD est mis en sortie
	return data;			// retourner la valeur
}


/*** Allume un pixel sur l'ecran a la position X(v),Y(h) ***/

/* 0<X<127 0<Y<63 */
void LCD_PutPixel(unsigned char X, unsigned char Y)
{
	unsigned char Val, page=Y>>3; 		// (Y/8) calcule la page courante
	LCD_GotoXY(X,page);				// Place le curseur sur l'octet courant
	Val=LCD_ReadData();				// Lit l'octet courant
	Val=LCD_ReadData();				// Lire une deuxieme fois !!! o_O ?
	
	Val|=0x01<<(Y-page*8);			// Dcale le pixel  la bonne coordonne

	LCD_GotoXY(X,page);
	LCD_Send_Data(Val);
}


void LCD_ClearPixel(unsigned char X, unsigned char Y)
{
	unsigned char Val,temp, page=Y>>3; 		// (Y/8) calcule la page courante
	LCD_GotoXY(X,page);				// Place le curseur sur l'octet courant
	Val=LCD_ReadData();				// Lit l'octet courant
	Val=LCD_ReadData();				// Lire une deuxieme fois !!! o_O ?
	
	temp=0x01<<(Y-page*8);		// slection du pixel
	temp=~temp;					// complmenter
	Val&=temp;					// mettre le pixel slectionner  zero

	LCD_GotoXY(X,page);
	LCD_Send_Data(Val);
}


void AfficheCible(unsigned char X, unsigned char Y)
{	unsigned char i,j,x;

		for (i=X;i<=(8+X);i++)	// Trace la 1re ligne
		LCD_PutPixel(i,Y);

		Y++;				// Passe  la ligne suivante

		for (x=0;x<6;x++)	// Rpter pour les lignes intermdiaires
		{
		LCD_PutPixel(X,Y);
		LCD_PutPixel(8+X,Y);
		Y++;
		}

		for (i=X;i<=(8+X);i++)	// Trace la dernire ligne
		LCD_PutPixel(i,Y);
}

/*** Idem que la fonction void AfficheCible mais avec des LCD_ClearPixel ***/
void EffaceCible(unsigned char X, unsigned char Y)
{	unsigned char i,j,x;

		for (i=X;i<=(8+X);i++)	// Trace la 1re ligne
		LCD_ClearPixel(i,Y);

		Y++;				// Passe  la ligne suivante

		for (x=0;x<(8-2);x++)	// Rpter pour les lignes intermdiaires
		{
		LCD_ClearPixel(X,Y);
		LCD_ClearPixel(8+X,Y);
		Y++;
		}

		for (i=X;i<=(8+X);i++)	// Trace la dernire ligne
		LCD_ClearPixel(i,Y);
}


/*** Trace une ligne Honrizontale sur le LCD ***/
void TraceLigneH(unsigned char Y, unsigned char X1,unsigned char X2)
{	unsigned char i;
	for (i=X1;i<X2;i++)		// de la colonne X1 a X2
	LCD_PutPixel(i,Y);		// Tracer la ligne Y
}

/*** Trace une ligne Verticale sur le LCD ***/
void TraceLigneV(unsigned char X, unsigned char Y1,unsigned char Y2)
{	unsigned char i;
	for (i=Y1;i<Y2;i++)		// de la ligne Y1 a Y2
	LCD_PutPixel(X,i);		// Tracer la colonne X
}

/*** Affiche un viseur en croix sur le LCD ***/
void AfficheViseur(unsigned char X, unsigned char Y)
{
	LCD_PutPixel(X,Y-1);
	LCD_PutPixel(X,Y-2);
	LCD_PutPixel(X,Y+1);
	LCD_PutPixel(X,Y+2);
	LCD_PutPixel(X-1,Y);
	LCD_PutPixel(X-2,Y);
	LCD_PutPixel(X+1,Y);
	LCD_PutPixel(X+2,Y);
}

void EffaceViseur(unsigned char X, unsigned char Y)
{
	LCD_ClearPixel(X,Y-1);
	LCD_ClearPixel(X,Y-2);
	LCD_ClearPixel(X,Y+1);
	LCD_ClearPixel(X,Y+2);
	LCD_ClearPixel(X-1,Y);
	LCD_ClearPixel(X-2,Y);
	LCD_ClearPixel(X+1,Y);
	LCD_ClearPixel(X+2,Y);
}

/*** Affiche le tir autour du viseur ***/
void AfficheTir(unsigned char X, unsigned char Y)
{
	LCD_PutPixel(X-3,Y-3);
	LCD_PutPixel(X-2,Y-2);
	LCD_PutPixel(X+3,Y-3);
	LCD_PutPixel(X+2,Y-2);
	LCD_PutPixel(X-3,Y+3);
	LCD_PutPixel(X-2,Y+2);
	LCD_PutPixel(X+3,Y+3);
	LCD_PutPixel(X+2,Y+2);

}

void EffaceTir(unsigned char X, unsigned char Y)
{
	LCD_ClearPixel(X-3,Y-3);
	LCD_ClearPixel(X-2,Y-2);
	LCD_ClearPixel(X+3,Y-3);
	LCD_ClearPixel(X+2,Y-2);
	LCD_ClearPixel(X-3,Y+3);
	LCD_ClearPixel(X-2,Y+2);
	LCD_ClearPixel(X+3,Y+3);
	LCD_ClearPixel(X+2,Y+2);
}


