// LCD.c
#include "GameGirl.h"		// inclure la librairie GameGirl.h
#include <p18f452.h>		// librairie du PIC
#include <delays.h>			// librairie de temporisation
#include "font_5x7.h"			// librairie des caractres

/*** Fonction de commande du LCD ***/
void LCD_Write_Command(unsigned char Data)
{	LCD_RS=0;				// dsactiver RS
	LCD_RW=0;				// dsactiver RW
	LCD_DATA=Data;			// placement de la donne sur le PORT
	Delay100TCYx(2);		// pendant 200 cyles
	LCD_Enable=1;			// activer Enable
	Delay100TCYx(2);		// pendant 200 cycles
	LCD_Enable=0;			// dsactiver Enable
}

/*** Fonction : initialisation du LCD ***/
void LCD_Init(void)
{	LCD_CSA=1;LCD_CSB=1;	// slection des deux contrleurs
	LCD_Enable=0;			// dsactiver Enable
	LCD_Reset=0;			// mettre Reset  0
	Delay100TCYx(1);		// pendant 100 cycles
	LCD_Reset=1;			// mettre Reset  1

	LCD_Write_Command(0x3F);	// instruction pour allumer l'afficheur
}

/*** Fonction : envoyer des donnes vers le LCD ***/
void LCD_Send_Data(unsigned char Data)
{	LCD_RS=1;				// activer RS
	LCD_RW=0;				// dsactiver RW
	LCD_DATA=Data;			// placement de la donne sur le PORT
	Delay100TCYx(2);		// pendant 200 cyles
	LCD_Enable=1;			// activer Enable
	Delay100TCYx(2);		// pendant 200 cycles
	LCD_Enable=0;			// dsactiver Enable
}

/*** Fonction : Effacer tout l'cran du LCD ***/
void LCD_Clear(void)
{	unsigned char i,x;
	LCD_CSA=1;						// slection du contrleur A
	LCD_CSB=1;						// slection du contrleur B
	
	for (x=0;x!=8;x++)			// rpter pour les 7 pages
	{
	LCD_Write_Command(0xb8 + x);	// slection de la page
	
		for (i=0;i!=64;i++)		// rpter pour les 64 octets
		LCD_Send_Data(0x00);	// envoyer un octet nul
	}
}

/*** Fonction : PLacer le curseur sur l'cran ***/
void LCD_GotoXY(unsigned char X,unsigned char Y)
{	if (X<64)
	{	
		LCD_CSA=0;
		LCD_CSB=1;			// slection du controleur B
		LCD_Write_Command(0xB8 + Y);	// slection de la page
		LCD_Write_Command(0x40 + X);	// slection de l'octet
	}
	else
	{	
		LCD_CSA=1;			// slection du controleur A
		LCD_CSB=0;
		LCD_Write_Command(0xB8 + Y);	// slection de la page
		LCD_Write_Command((0x40 + X)-64);	// slection de l'octet
	}
}

/*** Fonction : Afficher un caractre ASCII ***/
void LCD_Putchar(char ch)
{ char i;
	for(i=0;i!=5;i++)
		LCD_Send_Data(Font[((int)(ch-32)*5)+i]);	// Envoyer le caratre ASCII
		LCD_Send_Data(0x00);				// Envoyer un octet vide pour l'espace
	Delay10KTCYx(10);		// tempo
}

/*** Fonction : Lire l'tat de la mmoire du LCD ***/
char LCD_ReadData(void)
{ unsigned char data;
	TRISD = 0b11111111;		// le PortD est mis en entre
	LCD_RS=1;
	LCD_RW=1;
	LCD_Enable=1;
	Delay10TCYx(5);
	LCD_Enable=0;
	data=LCD_DATA;			// Lire la donne sur le port
	TRISD = 0b00000000;		// le PortD est mis en sortie
	return data;			// retourner la valeur
}


/*** Fonction : Allume un pixel sur l'ecran a la position X,Y ***/
/* 0<X<127 0<Y<63 */
void LCD_PutPixel(unsigned char X, unsigned char Y)
{
	unsigned char Val, page=Y>>3; 		// (Y/8) calcule la page courante
	LCD_GotoXY(X,page);				// Place le curseur sur l'octet courant
	Val=LCD_ReadData();				// Lit l'octet courant
	Val=LCD_ReadData();				// Lire une deuxieme fois !!! o_O ?
	
	Val|=0x01<<(Y-page*8);			// Dcale le pixel  la bonne coordonne

	LCD_GotoXY(X,page);			// Placer le curseur
	LCD_Send_Data(Val);			// Envoyer la donne
}

/*** Fonction : Efface un pixel sur l'ecran a la position X,Y ***/
void LCD_ClearPixel(unsigned char X, unsigned char Y)
{
	unsigned char Val,temp, page=Y>>3; 		// (Y/8) calcule la page courante
	LCD_GotoXY(X,page);				// Place le curseur sur l'octet courant
	Val=LCD_ReadData();				// Lit l'octet courant
	Val=LCD_ReadData();				// Lire une deuxieme fois !!! o_O ?
	
	temp=0x01<<(Y-page*8);		// slection du pixel
	temp=~temp;					// complmenter
	Val&=temp;					// mettre le pixel slectionn  zero

	LCD_GotoXY(X,page);			// Placer le curseur
	LCD_Send_Data(Val);			// Envoyer la donne
}


/*** Fonction : Affiche une cible reprsent par un carr vide de 8*8 pixels
		dont le pixel en haut a gauche est a la position X,Y 	***/
void AfficheCible(unsigned char X, unsigned char Y)
{	unsigned char i,j,x;

		for (i=X;i<=(8+X);i++)	// Trace la 1re ligne
		LCD_PutPixel(i,Y);

		Y++;				// Passe  la ligne suivante

		for (x=0;x<6;x++)	// Rpter pour les lignes intermdiaires
		{
		LCD_PutPixel(X,Y);
		LCD_PutPixel(8+X,Y);
		Y++;
		}

		for (i=X;i<=(8+X);i++)	// Trace la dernire ligne
		LCD_PutPixel(i,Y);
}

/*** Idem que la fonction void AfficheCible mais avec des LCD_ClearPixel ***/
void EffaceCible(unsigned char X, unsigned char Y)
{	unsigned char i,j,x;

		for (i=X;i<=(8+X);i++)	// Trace la 1re ligne
		LCD_ClearPixel(i,Y);

		Y++;				// Passe  la ligne suivante

		for (x=0;x<(8-2);x++)	// Rpter pour les lignes intermdiaires
		{
		LCD_ClearPixel(X,Y);
		LCD_ClearPixel(8+X,Y);
		Y++;
		}

		for (i=X;i<=(8+X);i++)	// Trace la dernire ligne
		LCD_ClearPixel(i,Y);
}


/*** Trace une ligne Horizontale sur le LCD ***/
void TraceLigneH(unsigned char Y, unsigned char X1,unsigned char X2)
{	unsigned char i;
	for (i=X1;i<X2;i++)		// de la colonne X1 a X2
	LCD_PutPixel(i,Y);		// Tracer la ligne Y
}

/*** Trace une ligne Verticale sur le LCD ***/
void TraceLigneV(unsigned char X, unsigned char Y1,unsigned char Y2)
{	unsigned char i;
	for (i=Y1;i<Y2;i++)		// de la ligne Y1 a Y2
	LCD_PutPixel(X,i);		// Tracer la colonne X
}

/*** Affiche un viseur en forme de croix sur le LCD a la position X,Y***/
void AfficheViseur(unsigned char X, unsigned char Y)
{
	LCD_PutPixel(X,Y-1);
	LCD_PutPixel(X,Y-2);
	LCD_PutPixel(X,Y+1);
	LCD_PutPixel(X,Y+2);
	LCD_PutPixel(X-1,Y);
	LCD_PutPixel(X-2,Y);
	LCD_PutPixel(X+1,Y);
	LCD_PutPixel(X+2,Y);
}

/*** Efface le viseur en forme de croix sur le LCD a la position X,Y***/
void EffaceViseur(unsigned char X, unsigned char Y)
{
	LCD_ClearPixel(X,Y-1);
	LCD_ClearPixel(X,Y-2);
	LCD_ClearPixel(X,Y+1);
	LCD_ClearPixel(X,Y+2);
	LCD_ClearPixel(X-1,Y);
	LCD_ClearPixel(X-2,Y);
	LCD_ClearPixel(X+1,Y);
	LCD_ClearPixel(X+2,Y);
}

/*** Affiche le tir autour du viseur ***/
void AfficheTir(unsigned char X, unsigned char Y)
{
	LCD_PutPixel(X-3,Y-3);
	LCD_PutPixel(X-2,Y-2);
	LCD_PutPixel(X+3,Y-3);
	LCD_PutPixel(X+2,Y-2);
	LCD_PutPixel(X-3,Y+3);
	LCD_PutPixel(X-2,Y+2);
	LCD_PutPixel(X+3,Y+3);
	LCD_PutPixel(X+2,Y+2);

}

/*** Efface le tir autour du viseur ***/
void EffaceTir(unsigned char X, unsigned char Y)
{
	LCD_ClearPixel(X-3,Y-3);
	LCD_ClearPixel(X-2,Y-2);
	LCD_ClearPixel(X+3,Y-3);
	LCD_ClearPixel(X+2,Y-2);
	LCD_ClearPixel(X-3,Y+3);
	LCD_ClearPixel(X-2,Y+2);
	LCD_ClearPixel(X+3,Y+3);
	LCD_ClearPixel(X+2,Y+2);
}



/*** AFFICHAGE DE CHAINES DE CARACTERES ***/

/*** Affiche l'cran d'intro ***/
void AfficheIntro(unsigned char T[], unsigned char A[], unsigned char R[], unsigned char N[]);
{	unsigned char i;
	TraceLigneH(12,0,128);	// Trace une ligne Horizontale
	TraceLigneH(38,0,128);	// idem

	LCD_GotoXY(10,3);
	for(i=0;i!=8;i++)
	LCD_Putchar(T[i]);	// Affiche le Titre

	LCD_GotoXY(80,5);
	for(i=0;i!=8;i++)
	LCD_Putchar(A[i]);	// Affiche l'auteur

	LCD_GotoXY(80,6);
	for(i=0;i!=8;i++)
	LCD_Putchar(R[i]);	// Affiche la rfrence

	LCD_GotoXY(10,7);
	for(i=0;i!=8;i++)
	LCD_Putchar(N[i]);	// Indique d'appuyer sur Bp Vert pour passer
}


/*** Affiche les commandes et les choix de vitesse ***/
void AfficheCommand_V(unsigned char B[], unsigned char R[], unsigned char V[]);
{	unsigned char i;
	LCD_GotoXY(2,1);
	for(i=0;i!=8;i++)
	LCD_Putchar(B[i]);	// Indique BP bleu pour tirer

	LCD_GotoXY(2,2);
	for(i=0;i!=8;i++)
	LCD_Putchar(R[i]);	// Indique BP bleu pour recharger

	LCD_GotoXY(2,4);
	for(i=0;i!=8;i++)
	LCD_Putchar(V[i]);		// Indique BP bleu pour Choisir la vitesse 1

	LCD_GotoXY(2,5);
	LCD_Putchar('V');		// Indique BP bleu pour Choisir la vitesse 2
	LCD_GotoXY(43,5);
	LCD_Putchar('2');

	LCD_GotoXY(2,6);
	LCD_Putchar('J');		// Indique BP bleu pour Choisir la vitesse 3
	LCD_GotoXY(43,6);
	LCD_Putchar('3');
}
 
/*** Trace le cadre et Affiche "Tirs" en bas a gauche de l'cran ***/
void AffCadres_Tirs(unsigned char T[]);
{	unsigned char i;
	TraceLigneH(0,0,128);	// Tracer un cadre
	TraceLigneH(54,0,128);	// ...
	TraceLigneV(0,0,54);	// ...
	TraceLigneV(127,0,54);	// ...

	LCD_GotoXY(0,7);
	for(i=0;i!=5;i++)
	LCD_Putchar(T[i]);	// Affiche "Tirs :" en bas a gauche du LCD
}



/*** AFFICHAGE DE VARIABLES ***/

/*** Affiche le nombre de tirs et le nombres de balles disponibles ***/
void AffTirs_Balles(unsigned t, unsigned char B);
{
		LCD_GotoXY(40,7);			// Affiche le nombre de Tirs en ASCII
		LCD_Putchar(((t/10)%10)+'0');		// Affiche les dizaines en ASCII
		LCD_GotoXY(48,7);
		LCD_Putchar((t%10)+'0');		// Affiche les units en ASCII

		LCD_GotoXY(100,7);			// Affiche le nombre de balles en ASCII
		LCD_Putchar(B+'0');
}



/*** AFFICHAGE DE CHAINES DE CARACTERES ET DE VARIBLES ***/

/*** Affiche le nombre de cibles passes ***/
void AffNbCibles(unsigned char C[], unsigned char NbC);
{	unsigned char i;
		LCD_GotoXY(10,2);
			for(i=0;i!=8;i++)
			LCD_Putchar(C[i]);			// Affiche "Cibles :"

		LCD_GotoXY(88,2); 				// Affiche le nombre de Cibles passes en ASCII
		LCD_Putchar(((NbC/10)%10)+'0');			// Affiche les dizaines en ASCII
		LCD_GotoXY(96,2);
		LCD_Putchar((NbC%10)+'0');			// Affiche les units en ASCII
}

/*** Affiche le nombre de tirs ***/
void AffNbTirs(unsigned char T[], unsigned char t);
{	unsigned char i;
		LCD_GotoXY(10,3);
			for(i=0;i!=8;i++)
			LCD_Putchar(T[i]);			// Affiche "Tirs :"

		LCD_GotoXY(88,3); 				// Affiche le nombre de Tirs en ASCII
		LCD_Putchar(((t/10)%10)+'0');			// Affiche les dizaines en ASCII
		LCD_GotoXY(96,3);
		LCD_Putchar((t%10)+'0');			// Affiche les units en ASCII
}

/*** Affiche le nombre de tirs russis ***/
void AffNbTirsOK(unsigned char TOK[], unsigned char TR);
{	unsigned char i;
		LCD_GotoXY(10,4);
			for(i=0;i!=8;i++)
			LCD_Putchar(TOK[i]);					// Affiche "Tirs OK :"

		LCD_GotoXY(88,4); 				// Affiche le nombre de TirsReussis en ASCII
		LCD_Putchar(((TR/10)%10)+'0');			// Affiche les dizaines en ASCII
		LCD_GotoXY(96,4);
		LCD_Putchar((TR%10)+'0');			// Affiche les units en ASCII
}

/*** Affiche le Score ***/
void AffScore(unsigned char S[], unsigned char s);
{	unsigned char i;
		LCD_GotoXY(10,6);
			for(i=0;i!=8;i++)
			LCD_Putchar(S[i]);					// Affiche "SCORE :"

		LCD_GotoXY(80,6); 						// Affiche le SCORE en ASCII
		LCD_Putchar(((s/100)+'0'));			// Affiche les centaines en ASCII
		LCD_GotoXY(88,6);
		LCD_Putchar(((s/10)%10)+'0'); 			// Affiche les dizaines en ASCII
		LCD_GotoXY(96,6);
		LCD_Putchar((s%10)+'0'); 			// Affiche les units en ASCII
}
